@@ -171,7 +171,7 @@ def step_edges(G, restriction_value):
             toRemove.append(edge)
     G.delete_edges(toRemove)  
 
-def step_spread(G, incubation_days, infection_duration, transmission_rate):
+def step_spread(G, incubation_days, infection_duration, transmission_rate, use_probabilities, gamma):
     """
     Make the infection spread across the network
     
@@ -189,11 +189,19 @@ def step_spread(G, incubation_days, infection_duration, transmission_rate):
     transmission_rate: float
         Value of the transmission rate for the disease in the network
 
+    use_probabilities: bool
+        Enables probabilities of being infected estimation
+
+    gamma: float
+        Parameter to regulate probability of being infected contact diffusion. Domain = (0, +inf). Higher values corresponds to stronger probability diffusion
+
     Return
     ------
     None
     
     """
+    if use_probabilities:
+        old_prob =  G.vs["prob_inf"]
 
     for node in G.vs:
         # update parameters if node is infected
@@ -209,6 +217,7 @@ def step_spread(G, incubation_days, infection_duration, transmission_rate):
                 if node["agent_status"] == "D":
                     node["quarantine"] = 0
                     node["test_result"] = - 1
+                    node["prob_inf"] = 0
                 
             # if it is still infective, spread the infection with his contacts
             if node["agent_status"] == "I":
@@ -248,8 +257,24 @@ def step_spread(G, incubation_days, infection_duration, transmission_rate):
                 
                 if random.uniform(0, 1) < 0.02:
                     node["needs_IC"] = True
+
+    # update prob of being infected
+    if use_probabilities:
+        nodes_contact_probs = [1] * len(list(G.vs))   
+        
+        for edge in G.es:
+            weight = edge["weight"]
+            source = edge.source
+            target = edge.target
+            nodes_contact_probs[source] *= 1 - old_prob[target] * (1 - np.e**(-gamma * weight))
+            nodes_contact_probs[target] *= 1 - old_prob[source] * (1 - np.e**(-gamma * weight))
+
+        for node in G.vs:
+            if node["agent_status"] != "D" and not (node["test_result"] == 0 and node["agent_status"] == "R"):
+                index = node.index
+                node["prob_inf"] = 1 - (1 - old_prob[index] * np.tanh(old_prob[index] + 0.5)) * nodes_contact_probs[index]
    
-def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing_efficiency):
+def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing_efficiency, use_probabilities, lambdaa):
     """
     Test some nodes of the network and put the in quarantine if needed
     
@@ -274,6 +299,12 @@ def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing
     contact_tracing_efficiency: float
         The percentage of contacts successfully traced
 
+    use_probabilities: bool
+        Enables probabilities of being infected estimation
+
+    lambdaa: float
+        Parameter to regulate influence of contacts with a positive. Domain = (0, 1). Higher values corresponds to stronger probability diffusion
+
     Return
     ------
     None
@@ -309,22 +340,18 @@ def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing
             node["test_result"] = 1
             node["quarantine"] = 14
             node["test_validity"] = 14
+            node["prob_inf"] = 1
             found_positive.add(node.index)
         else:
             node["test_result"] = 0
+            node["prob_inf"] = 0
             node["test_validity"] = incubation_days
 
+
+    to_test = list()
+
     if policy_test == "Random" and n_new_test > 0:
         to_test = random.sample(low_priority_test_pool, min(len(low_priority_test_pool), n_new_test))
-        for node in to_test:
-            if node["infected"]:
-                node["test_result"] = 1
-                node["quarantine"] = 14
-                node["test_validity"] = 14
-                found_positive.add(node.index)
-            else:
-                node["test_result"] = 0
-                node["test_validity"] = incubation_days
 
     if policy_test == "Degree Centrality" and n_new_test > 0:
         low_priority_test_pool_index = [x.index for x in low_priority_test_pool]
@@ -333,16 +360,7 @@ def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing
         sorted_pairs = sorted(zipped_lists, reverse = True)
         tuples = zip(*sorted_pairs)
         _, sorted_nodes = [ list(tuple) for tuple in  tuples]
-        for i in range(min(n_new_test, len(low_priority_test_pool))):
-            node = sorted_nodes[i]
-            if node["infected"]:
-                node["test_result"] = 1
-                node["quarantine"] = 14
-                node["test_validity"] = 14
-                found_positive.add(node.index)
-            else:
-                node["test_result"] = 0
-                node["test_validity"] = incubation_days
+        to_test = sorted_nodes[:min(n_new_test, len(low_priority_test_pool))]
 
     if policy_test == "Betweenness Centrality" and n_new_test > 0:
         low_priority_test_pool_index = [x.index for x in low_priority_test_pool]
@@ -353,16 +371,19 @@ def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing
         sorted_pairs = sorted(zipped_lists, reverse = True)
         tuples = zip(*sorted_pairs)
         _, sorted_nodes = [ list(tuple) for tuple in  tuples]
-        for i in range(min(n_new_test, len(low_priority_test_pool))):
-            node = sorted_nodes[i]
-            if node["infected"]:
-                node["test_result"] = 1
-                node["quarantine"] = 14
-                node["test_validity"] = 14
-                found_positive.add(node.index)
-            else:
-                node["test_result"] = 0
-                node["test_validity"] = incubation_days
+        to_test = sorted_nodes[:min(n_new_test, len(low_priority_test_pool))]
+
+    for node in to_test:
+        if node["infected"]:
+            node["test_result"] = 1
+            node["quarantine"] = 14
+            node["test_validity"] = 14
+            node["prob_inf"] = 1
+            found_positive.add(node.index)
+        else:
+            node["test_result"] = 0
+            node["prob_inf"] = 0
+            node["test_validity"] = incubation_days
     
     # to_quarantine will contain family contacts (quarantine 100%), 
     # possibly_quarantine will contain other contacts, quarantine influenced by contact tracing efficiency
@@ -393,99 +414,38 @@ def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing
             possibly_quarantine = list(possibly_quarantine)
         to_quarantine = list(to_quarantine) + possibly_quarantine
 
+        if use_probabilities:
+            # update prob of being infected of current contact 
+            for node in to_quarantine:
+                for contact in G.neighborhood(node)[1:]:
+                    contact_node = G.vs[contact]
+                    if contact_node["agent_status"] != "D" and not (contact_node["test_result"] == 0 and contact_node["agent_status"] == "R"):
+                        current_contact_weight = G[node, contact]
+                        contact_node["prob_inf"] = contact_node["prob_inf"] \
+                                                                        + lambdaa * np.e**(-(1 / current_contact_weight)) * (1 - contact_node["prob_inf"])
+            
+            # update prob of being infected of past tracked contact 
+                for net_index in range(len(nets)):
+                    net = nets[-net_index]
+                    for contact in net.neighborhood(node)[1:]:
+                        contact_node = G.vs[contact]
+                        if contact_node["agent_status"] != "D" and not (contact_node["test_result"] == 0 and contact_node["agent_status"] == "R"):
+                            if contact in to_quarantine:
+                                current_contact_weight = net[node, contact]
+                                contact_node["prob_inf"] = contact_node["prob_inf"] \
+                                                            + lambdaa * np.e**(- (net_index + 1) * (1 / current_contact_weight)) * (1 - contact_node["prob_inf"])
+
         to_quarantine = [G.vs[i] for i in to_quarantine]
 
         # put them in quarantine
-        # Need test?
         if to_quarantine != list() and to_quarantine != None:
             for node in to_quarantine:
-                node["quarantine"] = 14  
-
-
-'''
-def step_vaccine(G, n_vacc, policy_vacc, vacc_pool, agent_status_report):
-    """
-    Vaccine some individuals in the network
-    
-    Parameters
-    ----------
-    G: ig.Graph()
-        The contact network
-    
-    n_vacc: int
-        Number of vaccines avaiable that day
-    
-    policy_vacc: string
-        How to distribute the daily avaiable vaccines
-        Can be ["Random, Degree Centrality, Betweenness Centrality, Improved Degree Centrality, Improved Betweenness Centrality"]
-    
-    agent_status_report: list
-        Report of nodes status
-
-    Return
-    ------
-    agent_status_report: list
-        Updated report of nodes status
-
-    """
-
-    if len(vacc_pool) > 0:
-        
-        # random policy
-        if policy_vacc == "Random":
-            if n_vacc >= len(vacc_pool):
-                for index in vacc_pool:
-                    agent_status_report[index] = "V"
-                    G.nodes[index]["agent_status"] = "V"
-            else:
-                random_selected_to_vaccinate = random.sample(vacc_pool, n_vacc)
-                for index in random_selected_to_vaccinate:
-                    agent_status_report[index] = "V"
-                    G.nodes[index]["agent_status"] = "V"
-        
-        # degree policy
-        elif policy_vacc == "Degree Centrality":
-            degree_results = G.degree(vacc_pool)
-            order_by_degree = sorted(degree_results, key=lambda x: x[1], reverse = True)
-
-            for node, value in order_by_degree[:n_vacc]:
-                agent_status_report[node] = "V"
-                G.nodes[node]["agent_status"] = "V"
-
-        elif policy_vacc == "Improved Degree Centrality":
-            Z = G.subgraph(vacc_pool)
-            degree_results = Z.degree()
-            order_by_degree = sorted(degree_results, key=lambda x: x[1], reverse = True)
-
-            for node, value in order_by_degree[:n_vacc]:
-                agent_status_report[node] = "V"
-                G.nodes[node]["agent_status"] = "V"
-
-        # Betweenness policy
-        elif policy_vacc == "Betweenness Centrality":
-            betweenness_results = {node: val for node, val in ig.betweenness_centrality(G).items() if node in vacc_pool}
-
-            order_by_betweenness = sorted(betweenness_results, key=betweenness_results.get, reverse = True)
-            
-            for node in order_by_betweenness[:n_vacc]:
-                agent_status_report[node] = "V"
-                G.nodes[node]["agent_status"] = "V"
-
-        elif policy_vacc == "Improved Betweenness Centrality":
-            betweenness_results = {node: val for node, val in ig.betweenness_centrality_subset(G, sources = vacc_pool, targets = vacc_pool).items() if node in vacc_pool}
-
-            order_by_betweenness = sorted(betweenness_results, key=betweenness_results.get, reverse = True)
-            
-            for node in order_by_betweenness[:n_vacc]:
-                agent_status_report[node] = "V"
-                G.nodes[node]["agent_status"] = "V"
-    return agent_status_report  
-
-'''
+                node["quarantine"] = 14
 
 def step(G, step_index, incubation_days, infection_duration, transmission_rate,
          initial_day_restriction, restriction_duration, social_distance_strictness,
-         restriction_decreasing, nets, n_test, policy_test, contact_tracing_efficiency):
+         restriction_decreasing, nets, n_test, policy_test, contact_tracing_efficiency,
+         use_probabilities, gamma, lambdaa):
     """
     Advance the simulation of one step
     
@@ -532,6 +492,15 @@ def step(G, step_index, incubation_days, infection_duration, transmission_rate,
     contact_tracing_efficiency: float
         The percentage of contacts successfully traced
 
+    use_probabilities: bool
+        Enables probabilities of being infected estimation
+
+    gamma: float
+        Parameter to regulate probability of being infected contact diffusion. Domain = (0, +inf). Higher values corresponds to stronger probability diffusion
+
+    lambdaa: float
+        Parameter to regulate influence of contacts with a positive. Domain = (0, 1). Higher values corresponds to stronger probability diffusion
+
     Return
     ------
     G: ig.Graph()
@@ -556,17 +525,11 @@ def step(G, step_index, incubation_days, infection_duration, transmission_rate,
             step_edges(G, 1)
     
     # spread infection
-    step_spread(G, incubation_days, infection_duration, transmission_rate)
+    step_spread(G, incubation_days, infection_duration, transmission_rate, use_probabilities, gamma)
           
-    # make some test on nodes     
-    step_test(G, nets, incubation_days, n_test, policy_test, contact_tracing_efficiency)
+    # make some test on nodes
+    step_test(G, nets, incubation_days, n_test, policy_test, contact_tracing_efficiency, use_probabilities, lambdaa)
 
-    '''
-    # if avaiable, vaccinate some nodes
-    if step_index >= initial_day_vaccination:
-        vacc_pool = [i for i in range(len(agent_status_report)) if agent_status_report[i] == "S"]
-        agent_status_report = step_vaccine(G, n_vacc, policy_vacc, vacc_pool, agent_status_report)
-    '''
     agent_status_report = list()
     for node in G.vs:
         agent_status_report.append(node["agent_status"])
